@@ -1,11 +1,12 @@
 // in build.rs
-use clap_generate::{generate_to, generators::*};
+//use clap_generate::{generate_to, generators::*};
 include!("src/cli.rs");
 
 fn main() {
+    /*
+    // creates an error with cargo publish and then installing
     let mut app = Cli::into_app();
     let outdir = std::path::Path::new(env!("CARGO_MANIFEST_DIR")).join("target/");
-
     for bin_name in ["rb", "rustybam"] {
         // Bash, Fish, Zsh, PowerShell, Elvish
         generate_to(Bash, &mut app, bin_name, &outdir).unwrap();
@@ -14,4 +15,5 @@ fn main() {
         generate_to(PowerShell, &mut app, bin_name, &outdir).unwrap();
         generate_to(Elvish, &mut app, bin_name, &outdir).unwrap();
     }
+    */
 }
