@@ -70,29 +70,6 @@ impl<'a> Paf<'a> {
         paf.records = records;
         paf
     }
-    /*
-    pub fn populate_hash(&'a mut self) {
-        // read pafs into a hashtable based on name
-        for rec in &self.records[..] {
-            if self.records_by_contig.contains_key(&rec.t_name) {
-                self.records_by_contig
-                    .insert(rec.t_name.clone(), Vec::new());
-            }
-            self.records_by_contig
-                .get_mut(&rec.t_name)
-                .unwrap()
-                .push(&rec);
-        }
-    }
-
-    // calculate infor for different records
-    pub fn make_position_index(&mut self) {
-        eprintln!("Making paf index:");
-        self.records
-            .par_iter_mut()
-            .for_each(|rec| rec.aligned_pairs());
-    }
-    */
 }
 
 #[derive(Debug, Clone)]
@@ -201,7 +178,11 @@ impl PafRecord {
     pub fn aligned_pairs(&mut self) {
         let mut t_pos = self.t_st as i64 - 1;
         let mut q_pos = self.q_st as i64 - 1;
+
         let mut long_cigar = Vec::new();
+        self.tpos_aln = Vec::new();
+        self.qpos_aln = Vec::new();
+
         if self.strand == '-' {
             q_pos = self.q_en as i64; // ends are not inclusive
         }
@@ -231,23 +212,18 @@ impl PafRecord {
     }
 
     // Does a binary search on the alignment to find its index in the alignment
-    pub fn tpos_to_idx(&self, tpos: u64) -> usize {
-        /*eprintln!(
-            "{}<{}<{},  {}-{}",
-            self.t_st,
-            tpos,
-            self.t_en,
-            self.tpos_aln[0],
-            self.tpos_aln[self.tpos_aln.len() - 1]
-        );*/
-        let mut idx = self
-            .tpos_aln
-            .binary_search(&tpos)
-            .expect("must be within target alignment positions");
-        while idx + 1 < self.tpos_aln.len() && self.tpos_aln[idx + 1] == tpos {
-            idx += 1;
+    pub fn tpos_to_idx(&self, tpos: u64, right: bool) -> Result<usize, usize> {
+        let mut idx = self.tpos_aln.binary_search(&tpos)?;
+        if right {
+            while idx + 1 < self.tpos_aln.len() && self.tpos_aln[idx + 1] == tpos {
+                idx += 1;
+            }
+        } else {
+            while idx > 0 && self.tpos_aln[idx - 1] == tpos {
+                idx -= 1;
+            }
         }
-        idx
+        Ok(idx)
     }
 
     // subset cigar on coordiantes
@@ -287,36 +263,164 @@ impl PafRecord {
         self.t_en > rgn.st && self.t_st < rgn.en
     }
 
+    /// Return a tuple with the n of bases in the query and
+    /// target infered from the cigar string
+    pub fn infer_n_bases(&mut self) -> (u64, u64, u64, u64) {
+        let mut t_bases = 0;
+        let mut q_bases = 0;
+        let mut n_matches = 0;
+        let mut aln_len = 0;
+        for opt in self.cigar.into_iter() {
+            if consumes_reference(opt) {
+                t_bases += opt.len()
+            }
+            if consumes_query(opt) {
+                q_bases += opt.len()
+            }
+            if is_match(opt) {
+                n_matches += opt.len()
+            }
+            aln_len += opt.len();
+        }
+        (
+            t_bases as u64,
+            q_bases as u64,
+            n_matches as u64,
+            aln_len as u64,
+        )
+    }
+
     pub fn remove_trailing_indels(&mut self) {
-        let st_opt = *self.cigar.first().unwrap();
-        let en_opt = *self.cigar.last().unwrap();
+        //let (t_bases, q_bases, nmatch, aln_len) = self.infer_n_bases();
+        let cigar_len = self.cigar.len();
 
-        let mut remove_st = 0;
-        if matches!(st_opt, Ins(_) | Del(_)) {
-            remove_st = st_opt.len();
-            self.cigar = CigarString(self.cigar.0[1..].to_vec());
+        // find start to trim
+        let mut st_opt = *self.cigar.first().unwrap();
+        let mut remove_st_t = 0;
+        let mut remove_st_q = 0;
+        let mut remove_st_opts = 0;
+        let mut removed_st_opts = Vec::new();
+        while matches!(st_opt, Ins(_) | Del(_)) {
+            if matches!(st_opt, Del(_)) {
+                // consumes reference
+                remove_st_t += st_opt.len();
+                // TODO learn why I need this
+                remove_st_q += 1;
+            } else {
+                remove_st_q += st_opt.len();
+                // TODO learn why I need this
+                // TODO handle the case when it is a del and then and ins
+                remove_st_t += 1;
+            }
+            remove_st_opts += 1;
+            removed_st_opts.push(st_opt);
+            if remove_st_opts < cigar_len {
+                st_opt = self.cigar[remove_st_opts];
+            } else {
+                break;
+            }
         }
-        let mut remove_en = 0;
-        if matches!(en_opt, Ins(_) | Del(_)) {
-            remove_en = en_opt.len();
-            //self.cigar = CigarString(self.cigar.0[0..self.cigar.len() - 1].to_vec());
-            self.cigar.0.truncate(self.cigar.len() - 1);
+        // remove extra counts put in my the case of Del followed by Ins
+        if removed_st_opts.len() > 1 {
+            for i in 0..(removed_st_opts.len() - 1) {
+                let pre_opt = removed_st_opts[i];
+                let cur_opt = removed_st_opts[i + 1];
+                if matches!(pre_opt, Del(_)) && matches!(cur_opt, Ins(_)) {
+                    remove_st_t -= 1;
+                }
+            }
         }
 
-        if matches!(st_opt, Del(_)) || matches!(en_opt, Del(_)) {
-            self.t_st += remove_st as u64;
-            self.t_en -= remove_en as u64;
+        // find ends to trim
+        let mut en_opt = *self.cigar.last().unwrap();
+        let mut remove_en_t = 0;
+        let mut remove_en_q = 0;
+        let mut remove_en_opts = 0;
+        let mut removed_en_opts = Vec::new();
+        while matches!(en_opt, Ins(_) | Del(_)) {
+            if matches!(en_opt, Del(_)) {
+                // consumes reference
+                remove_en_t += en_opt.len();
+            } else {
+                remove_en_q += en_opt.len();
+            }
+            remove_en_opts += 1;
+            removed_en_opts.push(en_opt);
+            if cigar_len - remove_en_opts > 0 {
+                en_opt = self.cigar[cigar_len - 1 - remove_en_opts];
+            } else {
+                break;
+            }
         }
-        // change the qpos removal
+
+        // log that we did something
+        if remove_en_opts > 0 || remove_st_opts > 0 {
+            self.id += &format!(
+                "_TO.{}.{}",
+                CigarString(removed_st_opts),
+                CigarString(removed_en_opts)
+            );
+        }
+        // update the cigar string
+        self.cigar = CigarString(self.cigar.0[remove_st_opts..].to_vec());
+        self.cigar.0.truncate(self.cigar.len() - remove_en_opts);
+
+        // update the target coordiantes
+        self.t_st += remove_st_t as u64;
+        self.t_en -= remove_en_t as u64;
+
+        // update the query coordiantes if rc
         if self.strand == '-' {
-            std::mem::swap(&mut remove_st, &mut remove_en);
+            std::mem::swap(&mut remove_st_q, &mut remove_en_q);
         }
         // fix the query positions that need to be
-        if matches!(st_opt, Ins(_)) || matches!(en_opt, Ins(_)) {
-            self.q_st += remove_st as u64;
-            self.q_en -= remove_en as u64;
+        self.q_st += remove_st_q as u64;
+        self.q_en -= remove_en_q as u64;
+
+        // check we removed the indels
+        if self.cigar.len() > 0 {
+            let st_opt = *self.cigar.first().unwrap();
+            let en_opt = *self.cigar.last().unwrap();
+            if matches!(st_opt, Ins(_) | Del(_)) || matches!(en_opt, Ins(_) | Del(_)) {
+                eprintln!("Why are there still indels?\n{}", self);
+                //self.remove_trailing_indels();
+            }
         }
     }
+
+    pub fn check_integrity(&mut self) -> PafResult<()> {
+        let (t_bases, q_bases, nmatch, aln_len) = self.infer_n_bases();
+        if self.t_en - self.t_st != t_bases {
+            return Err(Error::PafParseCigar {
+                msg: format!(
+                    "target bases {} from cigar does not equal {}-{}={}\n{}\n",
+                    t_bases,
+                    self.t_en,
+                    self.t_st,
+                    self.t_en - self.t_st,
+                    self
+                ),
+            });
+        }
+        if self.q_en - self.q_st != q_bases {
+            return Err(Error::PafParseCigar {
+                msg: format!(
+                    "query bases {} from cigar does not equal {}-{}={}\n{}\n",
+                    q_bases,
+                    self.q_en,
+                    self.q_st,
+                    self.q_en - self.q_st,
+                    self
+                ),
+            });
+        }
+
+        // update other fields
+        self.nmatch = nmatch;
+        self.aln_len = aln_len;
+
+        Ok(())
+    }
 }
 
 impl fmt::Display for PafRecord {
@@ -360,6 +464,19 @@ pub fn consumes_query(cigar_opt: &Cigar) -> bool {
         Match(_i) | Ins(_i) | SoftClip(_i) | Diff(_i) | Equal(_i)
     )
 }
+
+/// # Example
+/// ```
+/// use rustybam::paf;
+/// use rust_htslib::bam::record::Cigar::*;
+/// assert!(paf::is_match(&Match(5)));
+/// assert!(paf::is_match(&Diff(5)));
+/// assert!(paf::is_match(&Equal(5)));
+/// ```
+pub fn is_match(cigar_opt: &Cigar) -> bool {
+    matches!(cigar_opt, Match(_i) | Diff(_i) | Equal(_i))
+}
+
 /// # Example
 /// ```
 /// use rustybam::paf;
@@ -467,13 +584,15 @@ pub fn paf_swap_query_and_target(paf: &PafRecord) -> PafRecord {
     // flip the index
     std::mem::swap(&mut flipped.qpos_aln, &mut flipped.tpos_aln);
 
-    if paf.strand == '-' {
-        flipped.qpos_aln.reverse();
-        flipped.tpos_aln.reverse();
-    }
     // flip the cigar
     flipped.cigar = cigar_swap_target_query(&paf.cigar, paf.strand);
     flipped.long_cigar = cigar_swap_target_query(&paf.long_cigar, paf.strand);
+
+    // update the alignment positions
+    if !flipped.tpos_aln.is_empty() {
+        flipped.aligned_pairs();
+    }
+
     flipped
 }
 
